//
//  OAVBAppDelegate.m
//  oldAllVoiBooks
//
//  Created by narita on 2017/05/18.
//  Copyright © 2017年 narita. All rights reserved.
//

#import "OAVBAppDelegate.h"
#import "HTMLReader.h"
#import "OAVBRSSItem.h"
#import "OAVBRSSChannel.h"

@interface OAVBAppDelegate ()

@end

@implementation OAVBAppDelegate

// HTML上の日付の文字列からNSDateへ変換する
- (NSDate*) dateFromDataString:(NSString*)inString
{
    // この形式でくる"2008年01月31日更新"
    NSString* theTemp = inString;
    
    theTemp = [theTemp stringByReplacingOccurrencesOfString:@"年"
                                                 withString:@"/"];

    theTemp = [theTemp stringByReplacingOccurrencesOfString:@"月"
                                                 withString:@"/"];

    theTemp = [theTemp stringByReplacingOccurrencesOfString:@"日更新"
                                                 withString:@""];
    
    NSDateFormatter* theFormatter = [[NSDateFormatter alloc] init];
    [theFormatter setDateFormat:@"yyyy/MM/dd"];
    [theFormatter setTimeZone:[NSTimeZone timeZoneForSecondsFromGMT:60 * 60 * 9]];
    
    return [theFormatter dateFromString:theTemp];
}

// NSDateからrss.xmlの<pubDate>Thu, 07 Apr 2011 05:00:00 +0900</pubDate>の形式の文字列へ変換する

- (NSString*) pubdateStringFromDate:(NSDate*)inDate
{
    NSDateFormatter* theFormatter = [[NSDateFormatter alloc] init];
    [theFormatter setDateFormat:@"EEE, dd MMM yyyy hh:mm:ss ZZ"];
    
    [theFormatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"en_US"]];

    [theFormatter setTimeZone:[NSTimeZone timeZoneForSecondsFromGMT:60 * 60 * 9]];

    
    return [theFormatter stringFromDate:inDate];
}

// ヴォイニッチの書棚_20100325の20100325の部分の文字列をだす
- (NSString*) titleStringFromDate:(NSDate*)inDate
{
    NSDateFormatter* theFormatter = [[NSDateFormatter alloc] init];
    [theFormatter setDateFormat:@"yyyyMMdd"];
    
    [theFormatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"en_US"]];
    
    [theFormatter setTimeZone:[NSTimeZone timeZoneForSecondsFromGMT:60 * 60 * 9]];
    
    
    return [theFormatter stringFromDate:inDate];
}

- (NSString*) urlStringFromNode:(HTMLElement*)inNode withURLString:(NSString*)inURLString
{
    NSString* theResult = inURLString;
    
    HTMLElement* theANode = [inNode firstNodeMatchingSelector:@"a"];
    
    if( theANode )
    {
        NSString* href = theANode[@"href"];

        NSURL* theURL = [NSURL URLWithString:href 
                               relativeToURL:[NSURL URLWithString:inURLString]];
        
        theResult = theURL.absoluteString;
    }
    
    return theResult;
}



- (NSArray<OAVBRSSItem*>*) itemsFromResourceName:(NSString*)inString
{
    NSMutableArray<OAVBRSSItem*>* theResult = @[].mutableCopy; 
    
    NSURL* theURL = [[NSBundle mainBundle] URLForResource:inString
                                            withExtension:@"html"];
    
    HTMLDocument *home = [HTMLDocument documentWithData:[NSData dataWithContentsOfURL:theURL]
                                      contentTypeHeader:nil];
    
    NSArray *nodes = [home nodesMatchingSelector:(@"table")];
    
    for (HTMLElement *node in nodes) 
    {
        NSString* border = node[@"border"];
        NSString* cellpadding = node[@"cellpadding"];
        NSString* cellspacing = node[@"cellspacing"];
        NSString* bgcolor = node[@"bgcolor"];
        NSString* width = node[@"width"];
        
        if( 
           [border isEqualToString:@"0"]      && 
           [cellpadding isEqualToString:@"2"] && 
           [cellspacing isEqualToString:@"2"] && 
           [bgcolor isEqualToString:@"white"] && 
           [width isEqualToString:@"100%"]
           ) 
        { 
            NSCharacterSet *whitespace = [NSCharacterSet whitespaceAndNewlineCharacterSet];
            
            NSArray<HTMLElement*>* theFontNodes = [node nodesMatchingSelector:(@"font")];
            
            if( theFontNodes.count > 3 )
            {
                OAVBRSSItem* theItem = [OAVBRSSItem new];
                
                // 更新日(NSDateに変換)
                NSDate* theDate = [self dateFromDataString:[theFontNodes[0].textContent stringByTrimmingCharactersInSet:whitespace]];                

                theItem.title = [NSString stringWithFormat:@"ヴォイニッチの書棚_%@", [self titleStringFromDate:theDate]];
                theItem.itunes_author = @"くりらじ";
                theItem.itunes_image = @"http://www.c-radio.net/images/voi-book.jpg";
                theItem.link = @"http://www.c-radio.net/20091119/voi-book.html";
                theItem.itemDescription = [theFontNodes[1].textContent stringByTrimmingCharactersInSet:whitespace];
                theItem.enclosure = [self urlStringFromNode:theFontNodes[3] withURLString:@"http://www.c-radio.net/xxx/"];
                theItem.pubDate = [self pubdateStringFromDate:theDate];
                
                [theResult addObject:theItem];
            }            
        } 
    } 

    
    
    return theResult.copy;
}



- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Insert code here to initialize your application
    
    OAVBRSSChannel* theChannel = [OAVBRSSChannel new];

    [theChannel.items addObjectsFromArray:[self itemsFromResourceName:@"voi-book_backnum_1"]];
    [theChannel.items addObjectsFromArray:[self itemsFromResourceName:@"voi-book_backnum_2"]];
    [theChannel.items addObjectsFromArray:[self itemsFromResourceName:@"voi-book_backnum_3"]];
    [theChannel.items addObjectsFromArray:[self itemsFromResourceName:@"voi-book_backnum_4"]];
    [theChannel.items addObjectsFromArray:[self itemsFromResourceName:@"voi-book_backnum_5"]];
    [theChannel.items addObjectsFromArray:[self itemsFromResourceName:@"voi-book_backnum_6"]];
    
//    NSLog(@"%@", theChannel.writeToString);
    
    [[NSPasteboard generalPasteboard] clearContents];
    [[NSPasteboard generalPasteboard] setString:theChannel.writeToString
                                        forType:NSPasteboardTypeString];
    
}


- (void)applicationWillTerminate:(NSNotification *)aNotification
{
    // Insert code here to tear down your application
}


@end
